@@ -1,7 +1,7 @@
 /* -*-c-*- ------------------ mix_ins.c :
  * Implementation of the functions declared in mix_ins.h
  * ------------------------------------------------------------------
- * Copyright (C) 1999, 2003, 2007 Free Software Foundation, Inc.
+ * Copyright (C) 1999, 2003, 2007, 2010 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -139,7 +139,7 @@ mix_ins_to_word (const mix_ins_t *ins)
 {
   g_return_val_if_fail (ins != NULL, MIX_WORD_ZERO);
   return  (mix_word_t)((ins->address<<18)|
-		       ((ins->index&7)<<12)|(ins->fspec<<6)|(ins->opcode));
+		       ((ins->index)<<12)|(ins->fspec<<6)|(ins->opcode));
 }
 
 mix_ins_id_t
@@ -261,7 +261,7 @@ void
 mix_ins_print (const mix_ins_t *ins)
 {
   g_return_if_fail (ins != NULL);
-  g_print (mix_get_string_from_id (mix_ins_id_from_ins (*ins)));
+  g_print ("%s", mix_get_string_from_id (mix_ins_id_from_ins (*ins)));
   g_print (" %s%d,%d(%d:%d)", mix_short_is_negative (ins->address) ? "-" : "+",
            mix_short_magnitude (ins->address), ins->index,
            mix_fspec_left (ins->fspec), mix_fspec_right (ins->fspec));
